#ifndef FrameCPP__COMMON__MEMORY_BUFFER_HH
#define FrameCPP__COMMON__MEMORY_BUFFER_HH

#if !defined(SWIGIMPORTED)
#include <list>
#include <iosfwd>
#endif /* !defined(SWIGIMPORTED) */

#include "general/types.hh"
#include "general/autoarray.hh"
#include "general/System.hh"

#include "framecpp/Common/FrameBufferInterface.hh"

namespace FrameCPP
{
  namespace Common
  {
    //===================================================================
    /// \brief Buffer appropriate for caching frame data
    ///
    /// If the buffer is for input, then the buffer needs to be
    /// initialized with the contents.
    /// The initialization needs to be done as via the call to the
    /// str( ) method before the buffer is associated with an
    /// IFrameStream object.
    /// \code
    /// MemoryBuffer	imb( std::ios:in);
    /// 
    /// imb.str( frame_file_data );
    /// 
    /// IFrameStream	ifs_mem( false, &imb );
    /// 
    /// \endcode
    ///
    //===================================================================
    class MemoryBuffer
      : public FrameBufferInterface,
	public std::stringbuf
    {
    public:
      typedef std::stringbuf		buffer_type;
      typedef buffer_type::char_type	char_type;

      //-----------------------------------------------------------------
      /// \brief Default constructor
      //-----------------------------------------------------------------
      MemoryBuffer( std::ios::openmode Mode,
		    bool ParentAutoDelete = true );

      //-----------------------------------------------------------------
      /// \brief Destructor
      //-----------------------------------------------------------------
      virtual ~MemoryBuffer( );

      //-----------------------------------------------------------------
      /// \brief adds a filter to the stream
      ///
      /// \param[in] Filter
      ///     Filter function to be added to the stream.
      //-----------------------------------------------------------------
      virtual void FilterAdd( FrameCPP::Common::StreamFilter* Filter );

      //-----------------------------------------------------------------
      /// \brief remote a filter to the stream
      ///
      /// \param[in] Filter
      ///     Filter function to be removed from the stream.
      //-----------------------------------------------------------------
      virtual void FilterRemove( FrameCPP::Common::StreamFilter* Filter );

      //-----------------------------------------------------------------
      /// \brief Returns true if filtering happens internally; false otherwise
      //-----------------------------------------------------------------
      bool FilterInternally( ) const;

    protected:
      friend class IStream;
      friend class OStream;

      //-----------------------------------------------------------------
      /// \brief Initialization of the buffer
      //-----------------------------------------------------------------
      virtual void buffer( );

    private:
      /// \brief Perform filtering operations on the stream
      virtual void filter( const char_type* Begin,
			   const char_type* End );

    };

  } // namespace - Common
} // namespace - FrameCPP

#if 0
#include "framecpp/Common/FrameBuffer.icc"
#endif /* 0 */

#endif /* FrameCPP__COMMON__MEMORY_BUFFER_HH */
